(function () {
  const Plugin = window.CKEditor5.core.Plugin;

  class CKEditor5LabelFixPlugin extends Plugin {
    static get pluginName() {
      return 'CKEditor5LabelFixPlugin';
    }

    init() {
      const editor = this.editor;
      const schema = editor.model.schema;
      const conversion = editor.conversion;

      // Register label as standard inline container
      schema.register('label', {
        inheritAllFrom: '$inlineObject',
        allowAttributes: [ 'for', 'class', 'style', 'lang', 'title' ]
      });

      // Allow inline children
      [ 'em', 'strong', 'b', 'i', 'span', 'cite', 'a', 'br', '$text' ].forEach(tag => {
        if (schema.isRegistered(tag)) {
          schema.extend(tag, { allowIn: 'label' });
        }
      });

      [ 'paragraph', 'listItem', 'li', 'span', '$root' ].forEach(parent => {
        if (schema.isRegistered(parent)) {
          schema.extend(parent, { allowChildren: [ 'label' ] });
        }
      });

      if (schema.isRegistered('cite')) {
        schema.extend('cite', { allowIn: [ 'a', 'label' ] });
      }

      if (schema.isRegistered('a')) {
        schema.extend('a', { allowChildren: [ 'cite' ] });
      }

      // Final elementToElement upcast, children handled automatically
      conversion.for('upcast').elementToElement({
        view: 'label',
        model: ( viewElement, { writer } ) =>
          writer.createElement( 'label', viewElement.getAttributes() ),
        converterPriority: 'high'
      });

      // Downcast
      conversion.for('downcast').elementToElement({
        model: 'label',
        view: ( modelElement, { writer } ) =>
          writer.createContainerElement( 'label', modelElement.getAttributes() )
      });

      editor.ui.componentFactory.add('cke5_label_fix_dummy', () => null);
    }
  }

  window.CKEditor5 = window.CKEditor5 || {};
  window.CKEditor5.ckeditor5_label_fix = {
    CKEditor5LabelFixPlugin: CKEditor5LabelFixPlugin
  };
})();
