<?php

namespace Drupal\ckeditor5_label_fix\Plugin\Filter;

use Drupal\filter\FilterProcessResult;
use Drupal\filter\Plugin\FilterBase;

/**
 * Fixes split <label> + <a> + <cite> elements caused by CKEditor5.
 *
 * @Filter(
 *   id = "label_link_fix_filter",
 *   title = @Translation("Fix CKEditor5 label/link nesting issue"),
 *   description = @Translation("Repairs label wrapping split over links and cite."),
 *   type = Drupal\filter\Plugin\FilterInterface::TYPE_TRANSFORM_IRREVERSIBLE,
 *   weight = 101,
 *   provider = "ckeditor5_label_fix",
 *   settings = {}
 * )
 */
class LabelLinkFixFilter extends FilterBase {

  /**
   * {@inheritdoc}
   */
  public function process($text, $langcode) {
    // Merge split label and anchor/cite content patterns.
    $pattern = '#<label([^>]*)>([^<]*?)</label>\s*(<a[^>]+>[^<]+(?:<cite>[^<]+</cite>)?</a>[^<]*)<label[^>]*>(.*?)</label>#i';

    $text = preg_replace_callback($pattern, function ($matches) {
      $label_attrs = $matches[1];
      $before_text = $matches[2];
      $link_html = $matches[3];
      $after_text = $matches[4];

      return '<label' . $label_attrs . '>' . $before_text . $link_html . $after_text . '</label>';
    }, $text);

    return new FilterProcessResult($text);
  }

}

